
#include "ods_interface.h"
using namespace std;

namespace AlgorithMan
{
    
    Open_Document_Spreadsheet::Open_Document_Spreadsheet()
    {
        Border = false;
    }
    
    Open_Document_Spreadsheet::Open_Document_Spreadsheet(string FileName)
    {
        Border = false;
        LoadFromFile(FileName);
    }
    
    int Open_Document_Spreadsheet::Sheets() const
    {
        return Content.size();
    }
    
    int Open_Document_Spreadsheet::Rows(int Sheet) const
    {
        if(Sheet >= Content.size())
            return 0;
        return Content[Sheet].size();
    }
    
    int Open_Document_Spreadsheet::Columns(int Sheet) const
    {
        if(Sheet >= Content.size())
            return 0;
        if(Content[Sheet].size() < 1)
            return 0;
        return Content[Sheet][0].size();
    }
    
    
    
    void Open_Document_Spreadsheet::AddSheet(int rows, int columns, string name)
    {
        Content.push_back(vector<vector<string> >(rows, vector<string>(columns,"")));
        SheetNames.push_back(name);
    }
    
    void Open_Document_Spreadsheet::AddRow(int Sheet)
    {
        Content[Sheet].push_back(vector<string>(Columns(Sheet),""));
    }
    
    void Open_Document_Spreadsheet::AddColumn(int Sheet)
    {
        for(vector<vector<string> >::iterator i = Content[Sheet].begin(); i != Content[Sheet].end(); i++)
            i->push_back("");
    }
    
    
    
    void Open_Document_Spreadsheet::LoadFromFile(string FileName)
    {
        Content.clear();
        string FullFileName = FileName;
        int i;
        for(i = FileName.size()-1; i > 0; i--)
        {
            if(FileName[i] == '/')
            {
                FileName.erase(0,i+1);
                break;
            }
        }
            
        // extract ods file to temporary location (ods is a zip archive)
        string tmp = "/tmp";
        mkdir((tmp+"/ods_interface").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName).c_str(),0711);
        int x = system((string("unzip -qq -u -d ")+tmp+"/ods_interface/"+FileName+" "+FullFileName).c_str());
        
        // load content.xml file
        ifstream content_xml((tmp+"/ods_interface/"+FileName+"/content.xml").c_str());
        XML xml;
        content_xml >> xml;
        content_xml.close();
        
        // traverse the XML structure
        list<XML*> doc = xml.FindChildren("office:document-content");
        for(list<XML*>::iterator d = doc.begin(); d != doc.end(); d++)
        {
            list<XML*> body = (*d)->FindChildren("office:body");
            for(list<XML*>::iterator b = body.begin(); b != body.end(); b++)
            {
                list<XML*> spreadsheet = (*b)->FindChildren("office:spreadsheet");
                for(list<XML*>::iterator s = spreadsheet.begin(); s != spreadsheet.end(); s++)
                {
                    list<XML*> table = (*s)->FindChildren("table:table");
                    for(list<XML*>::iterator t = table.begin(); t != table.end(); t++)
                    {
                        Content.push_back(vector<vector<string > >(0,vector<string>(0,"")));
                        SheetNames.push_back((*t)->GetAttribute("table:name"));
                        list<XML*> row = (*t)->FindChildren("table:table-row");
                        for(list<XML*>::iterator r = row.begin(); r != row.end(); r++)
                        {
                            Content.back().push_back(vector<string>(0,""));
                            list<XML*> cell = (*r)->FindChildren("table:table-cell");
                            for(list<XML*>::iterator c = cell.begin(); c != cell.end(); c++)
                            {
                                std::string srepeated = (*c)->GetAttribute("table:number-columns-repeated", "1");
                                stringstream s;
                                s << srepeated;
                                int repeated;
                                s >> repeated;
                                
                                for(int i = 0; i < repeated; i++)
                                {
                                    Content.back().back().push_back("");
                                    list<XML*> text = (*c)->FindChildren("text:p");
                                    for(list<XML*>::iterator x = text.begin(); x != text.end(); x++)
                                    {
                                        for(list<XML_Element*>::iterator p = (*x)->children.begin(); p != (*x)->children.end(); p++)
                                        {
                                            XML_Text *out = dynamic_cast<XML_Text*>(*p);
                                            if(out != NULL)
                                            {
                                                if(out->text.size() > 0)
                                                    Content.back().back().back()=*(out->text.begin());
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        for(vector<vector<vector<string> > >::iterator s = Content.begin(); s != Content.end(); s++)
        {
            int maxwidth=0;
            for(vector<vector<string> >::iterator r = s->begin(); r != s->end(); r++)
                if(r->size() > maxwidth)
                    maxwidth = r->size();
            for(vector<vector<string> >::iterator r = s->begin(); r != s->end(); r++)
                if(r->size() < maxwidth)
                    r->resize(maxwidth);
        }

        x = system((string("rm -rf ")+tmp+"/ods_interface/"+FileName).c_str());
    }

    void Open_Document_Spreadsheet::SaveToFile(string FileName) const
    {
        string FullFileName = FileName;
        int i;
        for(i = FileName.size()-1; i > 0; i--)
        {
            if(FileName[i] == '/')
            {
                FileName.erase(0,i+1);
                break;
            }
        }

        string tmp = "/tmp";
        mkdir((tmp+"/ods_interface").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName).c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Configurations2").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Configurations2/accelerator").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Configurations2/floater").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Configurations2/images").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Configurations2/images/Bitmaps").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Configurations2/menubar").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Configurations2/popupmenu").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Configurations2/progressbar").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Configurations2/statusbar").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Configurations2/toolbar").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/META-INF").c_str(),0711);
        mkdir((tmp+"/ods_interface/"+FileName+"/Thumbnails").c_str(),0711);

        ofstream current((tmp+"/ods_interface/"+FileName+"/Configurations2/accelerator/current.xml").c_str());
        current.close();

        ofstream manifest((tmp+"/ods_interface/"+FileName+"/META-INF/manifest.xml").c_str());
        manifest << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        manifest << "<manifest:manifest xmlns:manifest=\"urn:oasis:names:tc:opendocument:xmlns:manifest:1.0\">\n";
        manifest << " <manifest:file-entry manifest:media-type=\"application/vnd.oasis.opendocument.spreadsheet\" manifest:version=\"1.2\" manifest:full-path=\"/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Configurations2/statusbar/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Configurations2/accelerator/current.xml\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Configurations2/accelerator/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Configurations2/floater/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Configurations2/popupmenu/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Configurations2/progressbar/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Configurations2/menubar/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Configurations2/toolbar/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Configurations2/images/Bitmaps/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Configurations2/images/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"application/vnd.sun.xml.ui.configuration\" manifest:full-path=\"Configurations2/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"content.xml\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"styles.xml\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"meta.xml\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"\" manifest:full-path=\"Thumbnails/\"/>\n";
        manifest << " <manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"settings.xml\"/>\n";
        manifest << "</manifest:manifest>\n";
        manifest.close();
        
        ofstream styles_xml((tmp+"/ods_interface/"+FileName+"/styles.xml").c_str());
        styles_xml << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        styles_xml << "    <office:document-styles xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\"\n";
        styles_xml << "        xmlns:style=\"urn:oasis:names:tc:opendocument:xmlns:style:1.0\"\n";
        styles_xml << "        xmlns:text=\"urn:oasis:names:tc:opendocument:xmlns:text:1.0\"\n";
        styles_xml << "        xmlns:table=\"urn:oasis:names:tc:opendocument:xmlns:table:1.0\"\n";
        styles_xml << "        xmlns:draw=\"urn:oasis:names:tc:opendocument:xmlns:drawing:1.0\"\n";
        styles_xml << "        xmlns:fo=\"urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0\"\n";
        styles_xml << "        xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n";
        styles_xml << "        xmlns:dc=\"http://purl.org/dc/elements/1.1/\"\n";
        styles_xml << "        xmlns:meta=\"urn:oasis:names:tc:opendocument:xmlns:meta:1.0\"\n";
        styles_xml << "        xmlns:number=\"urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0\"\n";
        styles_xml << "        xmlns:presentation=\"urn:oasis:names:tc:opendocument:xmlns:presentation:1.0\"\n";
        styles_xml << "        xmlns:svg=\"urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0\"\n";
        styles_xml << "        xmlns:chart=\"urn:oasis:names:tc:opendocument:xmlns:chart:1.0\"\n";
        styles_xml << "        xmlns:dr3d=\"urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0\"\n";
        styles_xml << "        xmlns:math=\"http://www.w3.org/1998/Math/MathML\"\n";
        styles_xml << "        xmlns:form=\"urn:oasis:names:tc:opendocument:xmlns:form:1.0\"\n";
        styles_xml << "        xmlns:script=\"urn:oasis:names:tc:opendocument:xmlns:script:1.0\"\n";
        styles_xml << "        xmlns:ooo=\"http://openoffice.org/2004/office\"\n";
        styles_xml << "        xmlns:ooow=\"http://openoffice.org/2004/writer\"\n";
        styles_xml << "        xmlns:oooc=\"http://openoffice.org/2004/calc\"\n";
        styles_xml << "        xmlns:dom=\"http://www.w3.org/2001/xml-events\"\n";
        styles_xml << "        xmlns:rpt=\"http://openoffice.org/2005/report\"\n";
        styles_xml << "        xmlns:of=\"urn:oasis:names:tc:opendocument:xmlns:of:1.2\"\n";
        styles_xml << "        xmlns:rdfa=\"http://docs.oasis-open.org/opendocument/meta/rdfa#\"\n";
        styles_xml << "        office:version=\"1.2\">\n";
        styles_xml << "        <office:font-face-decls>\n";
        styles_xml << "            <style:font-face style:name=\"Liberation Sans\" svg:font-family=\"&apos;Liberation Sans&apos;\" style:font-family-generic=\"swiss\" style:font-pitch=\"variable\"/>\n";
        styles_xml << "            <style:font-face style:name=\"DejaVu Sans\" svg:font-family=\"&apos;DejaVu Sans&apos;\" style:font-family-generic=\"system\" style:font-pitch=\"variable\"/>\n";
        styles_xml << "        </office:font-face-decls>\n";
        styles_xml << "        \n";
        styles_xml << "        <office:styles>\n";
        styles_xml << "            <style:default-style style:family=\"table-cell\">\n";
        styles_xml << "                <style:table-cell-properties style:decimal-places=\"2\"/>\n";
        styles_xml << "                <style:paragraph-properties style:tab-stop-distance=\"1.25cm\"/>\n";
        styles_xml << "                <style:text-properties style:font-name=\"Liberation Sans\"\n";
        styles_xml << "                    fo:language=\"pt\" fo:country=\"BR\" style:font-name-asian=\"DejaVu Sans\"\n";
        styles_xml << "                    style:language-asian=\"zxx\" style:country-asian=\"none\"\n";
        styles_xml << "                    style:font-name-complex=\"DejaVu Sans\" style:language-complex=\"zxx\"\n";
        styles_xml << "                    style:country-complex=\"none\"/>\n";
        styles_xml << "            </style:default-style>\n";
        styles_xml << "            <number:number-style style:name=\"N0\">\n";
        styles_xml << "                <number:number number:min-integer-digits=\"1\"/>\n";
        styles_xml << "            </number:number-style>\n";
        styles_xml << "            <number:currency-style style:name=\"N108P0\" style:volatile=\"true\">\n";
        styles_xml << "                <number:currency-symbol number:language=\"pt\" number:country=\"BR\">R$</number:currency-symbol>\n";
        styles_xml << "                <number:text></number:text>\n";
        styles_xml << "                <number:number number:decimal-places=\"2\" number:min-integer-digits=\"1\" number:grouping=\"true\"/>\n";
        styles_xml << "            </number:currency-style>\n";
        styles_xml << "            <number:currency-style style:name=\"N108\">\n";
        styles_xml << "                <style:text-properties fo:color=\"#ff0000\"/>\n";
        styles_xml << "                <number:text>-</number:text>\n";
        styles_xml << "                <number:currency-symbol number:language=\"pt\" number:country=\"BR\">R$</number:currency-symbol>\n";
        styles_xml << "                <number:text> </number:text>\n";
        styles_xml << "                <number:number number:decimal-places=\"2\" number:min-integer-digits=\"1\" number:grouping=\"true\"/>\n";
        styles_xml << "                <style:map style:condition=\"value()&gt;=0\" style:apply-style-name=\"N108P0\"/>\n";
        styles_xml << "            </number:currency-style>\n";
        styles_xml << "            <number:currency-style style:name=\"N106P0\" style:volatile=\"true\">\n";
        styles_xml << "                <number:number number:decimal-places=\"2\" number:min-integer-digits=\"1\" number:grouping=\"true\"/>\n";
        styles_xml << "                <number:text> </number:text>\n";
        styles_xml << "                <number:currency-symbol number:language=\"de\" number:country=\"DE\">€</number:currency-symbol>\n";
        styles_xml << "            </number:currency-style>\n";
        styles_xml << "            <number:currency-style style:name=\"N106\">\n";
        styles_xml << "                <style:text-properties fo:color=\"#ff0000\"/>\n";
        styles_xml << "                <number:text>-</number:text>\n";
        styles_xml << "                <number:number number:decimal-places=\"2\" number:min-integer-digits=\"1\" number:grouping=\"true\"/>\n";
        styles_xml << "                <number:text> </number:text>\n";
        styles_xml << "                <number:currency-symbol number:language=\"de\" number:country=\"DE\">€</number:currency-symbol>\n";
        styles_xml << "                <style:map style:condition=\"value()&gt;=0\" style:apply-style-name=\"N106P0\"/>\n";
        styles_xml << "            </number:currency-style>\n";
        styles_xml << "            <style:style style:name=\"Default\" style:family=\"table-cell\"/>\n";
        styles_xml << "            <style:style style:name=\"Result\" style:family=\"table-cell\" style:parent-style-name=\"Default\">\n";
        styles_xml << "                <style:text-properties fo:font-style=\"italic\" style:text-underline-style=\"solid\" style:text-underline-width=\"auto\" style:text-underline-color=\"font-color\" fo:font-weight=\"bold\"/>\n";
        styles_xml << "            </style:style>\n";
        styles_xml << "            <style:style style:name=\"Result2\" style:family=\"table-cell\" style:parent-style-name=\"Result\" style:data-style-name=\"N108\"/>\n";
        styles_xml << "            <style:style style:name=\"Heading\" style:family=\"table-cell\" style:parent-style-name=\"Default\">\n";
        styles_xml << "                <style:table-cell-properties style:text-align-source=\"fix\" style:repeat-content=\"false\"/>\n";
        styles_xml << "                <style:paragraph-properties fo:text-align=\"center\"/>\n";
        styles_xml << "                <style:text-properties fo:font-size=\"16pt\" fo:font-style=\"italic\" fo:font-weight=\"bold\"/>\n";
        styles_xml << "                </style:style>\n";
        styles_xml << "                <style:style style:name=\"Heading1\" style:family=\"table-cell\" style:parent-style-name=\"Heading\">\n";
        styles_xml << "                <style:table-cell-properties style:rotation-angle=\"90\"/>\n";
        styles_xml << "            </style:style>\n";
        styles_xml << "        </office:styles>\n";
        styles_xml << "        <office:automatic-styles>\n";
        styles_xml << "            <style:page-layout style:name=\"Mpm1\">\n";
        styles_xml << "                <style:page-layout-properties style:writing-mode=\"lr-tb\"/>\n";
        styles_xml << "                <style:header-style>\n";
        styles_xml << "                    <style:header-footer-properties fo:min-height=\"0.751cm\" fo:margin-left=\"0cm\" fo:margin-right=\"0cm\" fo:margin-bottom=\"0.25cm\"/>\n";
        styles_xml << "                </style:header-style>\n";
        styles_xml << "                <style:footer-style>\n";
        styles_xml << "                    <style:header-footer-properties fo:min-height=\"0.751cm\" fo:margin-left=\"0cm\" fo:margin-right=\"0cm\" fo:margin-top=\"0.25cm\"/>\n";
        styles_xml << "                </style:footer-style>\n";
        styles_xml << "            </style:page-layout>\n";
        styles_xml << "            <style:page-layout style:name=\"Mpm2\">\n";
        styles_xml << "                <style:page-layout-properties style:writing-mode=\"lr-tb\"/>\n";
        styles_xml << "                <style:header-style>\n";
        styles_xml << "                    <style:header-footer-properties fo:min-height=\"0.751cm\" fo:margin-left=\"0cm\" fo:margin-right=\"0cm\" fo:margin-bottom=\"0.25cm\" fo:border=\"0.088cm solid #000000\" fo:padding=\"0.018cm\" fo:background-color=\"#c0c0c0\">\n";
        styles_xml << "                        <style:background-image/>\n";
        styles_xml << "                    </style:header-footer-properties>\n";
        styles_xml << "                </style:header-style>\n";
        styles_xml << "                <style:footer-style>\n";
        styles_xml << "                    <style:header-footer-properties fo:min-height=\"0.751cm\" fo:margin-left=\"0cm\" fo:margin-right=\"0cm\" fo:margin-top=\"0.25cm\" fo:border=\"0.088cm solid #000000\" fo:padding=\"0.018cm\" fo:background-color=\"#c0c0c0\">\n";
        styles_xml << "                        <style:background-image/>\n";
        styles_xml << "                    </style:header-footer-properties>\n";
        styles_xml << "                </style:footer-style>\n";
        styles_xml << "            </style:page-layout>\n";
        styles_xml << "        </office:automatic-styles>\n";
        styles_xml << "        <office:master-styles>\n";
        styles_xml << "            <style:master-page style:name=\"Default\" style:page-layout-name=\"Mpm1\">\n";
        styles_xml << "                <style:header>\n";
        styles_xml << "                    <text:p>\n";
        //                                                       the ??? are seperated to suppress a compiler warning
        styles_xml << "                        <text:sheet-name>"<<'?'<<'?'<<'?'<<"</text:sheet-name>\n";
        styles_xml << "                    </text:p>\n";
        styles_xml << "                </style:header>\n";
        styles_xml << "                <style:header-left style:display=\"false\"/>\n";
        styles_xml << "                <style:footer>\n";
        styles_xml << "                    <text:p>Page <text:page-number>1</text:page-number></text:p>\n";
        styles_xml << "                </style:footer>\n";
        styles_xml << "                <style:footer-left style:display=\"false\"/>\n";
        styles_xml << "            </style:master-page>\n";
        styles_xml << "            <style:master-page style:name=\"Report\" style:page-layout-name=\"Mpm2\">\n";
        styles_xml << "                <style:header>\n";
        styles_xml << "                    <style:region-left>\n";
        //                                                               the ??? are seperated to suppress a compiler warning
        styles_xml << "                        <text:p><text:sheet-name>"<<'?'<<'?'<<'?'<<"</text:sheet-name> (<text:title>"<<'?'<<'?'<<'?'<<"</text:title>)</text:p>\n";
        styles_xml << "                    </style:region-left>\n";
        styles_xml << "                    <style:region-right>\n";
        styles_xml << "                        <text:p>\n";
        styles_xml << "                            <text:date style:data-style-name=\"N2\" text:date-value=\"2009-11-27\">27.11.2009</text:date>, <text:time>10:22:30</text:time>\n";
        styles_xml << "                        </text:p>\n";
        styles_xml << "                    </style:region-right>\n";
        styles_xml << "                </style:header>\n";
        styles_xml << "                <style:header-left style:display=\"false\"/>\n";
        styles_xml << "                <style:footer>\n";
        styles_xml << "                    <text:p>Page <text:page-number>1</text:page-number> / <text:page-count>99</text:page-count></text:p>\n";
        styles_xml << "                </style:footer>\n";
        styles_xml << "                <style:footer-left style:display=\"false\"/>\n";
        styles_xml << "            </style:master-page>\n";
        styles_xml << "        </office:master-styles>\n";
        styles_xml << "    </office:document-styles>\n";
        styles_xml.close();
        
        ofstream mimetype((tmp+"/ods_interface/"+FileName+"/mimetype").c_str());
        mimetype << "application/vnd.oasis.opendocument.spreadsheet";
        mimetype.close();

        ofstream meta_xml((tmp+"/ods_interface/"+FileName+"/meta.xml").c_str());
        meta_xml << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        meta_xml << "<office:document-meta xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\"\n";
        meta_xml << "    xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n";
        meta_xml << "    xmlns:dc=\"http://purl.org/dc/elements/1.1/\"\n";
        meta_xml << "    xmlns:meta=\"urn:oasis:names:tc:opendocument:xmlns:meta:1.0\"\n";
        meta_xml << "    xmlns:ooo=\"http://openoffice.org/2004/office\"\n";
        meta_xml << "    office:version=\"1.2\">\n";
        meta_xml << "    <office:meta>\n";
        
//        meta_xml << "        <meta:generator>OpenOffice.org/3.1$Linux OpenOffice.org_project/310m19$Build-9420</meta:generator>\n";
//        meta_xml << "        <meta:creation-date>2008-03-26T16:46:20</meta:creation-date>\n";
        meta_xml << "        <meta:generator>ods_interface by AlgorithMan</meta:generator>\n";
        meta_xml << "        <meta:creation-date>2009-11-29T19:47</meta:creation-date>\n";

        meta_xml << "        <meta:editing-cycles>1</meta:editing-cycles>\n";
        meta_xml << "        <meta:editing-duration>PT00H00M30S</meta:editing-duration>\n";
        
        time_t rawtime;
        time(&rawtime);
        struct tm *timeinfo = localtime(&rawtime);
        char buffer[21];
        strftime(buffer, 21, "%y-%m-%dT%H:%M:%S", timeinfo);
        meta_xml << "        <dc:date>"<<buffer<<"</dc:date>\n";

        int cells = 0;
        for(vector<vector<vector<string> > >::const_iterator it = Content.begin(); it != Content.end(); it++)
            cells += it->size() * (*it)[0].size();
        meta_xml << "        <meta:document-statistic meta:table-count=\""<<Content.size()<<"\" meta:cell-count=\""<<cells<<"\" meta:object-count=\"0\"/>\n";

        
        meta_xml << "        <meta:user-defined meta:name=\"Info 1\"/>\n";
        meta_xml << "        <meta:user-defined meta:name=\"Info 2\"/>\n";
        meta_xml << "        <meta:user-defined meta:name=\"Info 3\"/>\n";
        meta_xml << "        <meta:user-defined meta:name=\"Info 4\"/>\n";
        meta_xml << "    </office:meta>\n";
        meta_xml << "</office:document-meta>\n";
        meta_xml.close();
        
        ofstream settings_xml((tmp+"/ods_interface/"+FileName+"/settings.xml").c_str());
        settings_xml << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        settings_xml << "<office:document-settings xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\"\n";
        settings_xml << "xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:config=\"urn:oasis:names:tc:opendocument:xmlns:config:1.0\"\n";
        settings_xml << "xmlns:ooo=\"http://openoffice.org/2004/office\" office:version=\"1.2\">\n";
        settings_xml << "    <office:settings>\n";
        settings_xml << "        <config:config-item-set config:name=\"ooo:view-settings\">\n";
        settings_xml << "            <config:config-item config:name=\"VisibleAreaTop\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"VisibleAreaLeft\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"VisibleAreaWidth\" config:type=\"int\">9032</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"VisibleAreaHeight\" config:type=\"int\">899</config:config-item>\n";
        settings_xml << "            <config:config-item-map-indexed config:name=\"Views\">\n";
        settings_xml << "                <config:config-item-map-entry>\n";
        settings_xml << "                    <config:config-item config:name=\"ViewId\" config:type=\"string\">View1</config:config-item>\n";
        settings_xml << "                    <config:config-item-map-named config:name=\"Tables\">\n";
        settings_xml << "                        <config:config-item-map-entry config:name=\"Sheet1\">\n";
        settings_xml << "                            <config:config-item config:name=\"CursorPositionX\" config:type=\"int\">3</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"CursorPositionY\" config:type=\"int\">2</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"HorizontalSplitMode\" config:type=\"short\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"VerticalSplitMode\" config:type=\"short\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"HorizontalSplitPosition\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"VerticalSplitPosition\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"ActiveSplitRange\" config:type=\"short\">2</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"PositionLeft\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"PositionRight\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"PositionTop\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"PositionBottom\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"ZoomType\" config:type=\"short\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"ZoomValue\" config:type=\"int\">100</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"PageViewZoomValue\" config:type=\"int\">60</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"ShowGrid\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "                        </config:config-item-map-entry>\n";
        settings_xml << "                        <config:config-item-map-entry config:name=\"Sheet2\">\n";
        settings_xml << "                            <config:config-item config:name=\"CursorPositionX\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"CursorPositionY\" config:type=\"int\">1</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"HorizontalSplitMode\" config:type=\"short\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"VerticalSplitMode\" config:type=\"short\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"HorizontalSplitPosition\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"VerticalSplitPosition\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"ActiveSplitRange\" config:type=\"short\">2</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"PositionLeft\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"PositionRight\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"PositionTop\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"PositionBottom\" config:type=\"int\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"ZoomType\" config:type=\"short\">0</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"ZoomValue\" config:type=\"int\">100</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"PageViewZoomValue\" config:type=\"int\">60</config:config-item>\n";
        settings_xml << "                            <config:config-item config:name=\"ShowGrid\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "                        </config:config-item-map-entry>\n";
        settings_xml << "                    </config:config-item-map-named>\n";
        settings_xml << "                    <config:config-item config:name=\"ActiveTable\" config:type=\"string\">Sheet2</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"HorizontalScrollbarWidth\" config:type=\"int\">270</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"ZoomType\" config:type=\"short\">0</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"ZoomValue\" config:type=\"int\">100</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"PageViewZoomValue\" config:type=\"int\">60</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"ShowPageBreakPreview\" config:type=\"boolean\">false</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"ShowZeroValues\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"ShowNotes\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"ShowGrid\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"GridColor\" config:type=\"long\">12632256</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"ShowPageBreaks\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"HasColumnRowHeaders\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"HasSheetTabs\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"IsOutlineSymbolsSet\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"IsSnapToRaster\" config:type=\"boolean\">false</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"RasterIsVisible\" config:type=\"boolean\">false</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"RasterResolutionX\" config:type=\"int\">1000</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"RasterResolutionY\" config:type=\"int\">1000</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"RasterSubdivisionX\" config:type=\"int\">1</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"RasterSubdivisionY\" config:type=\"int\">1</config:config-item>\n";
        settings_xml << "                    <config:config-item config:name=\"IsRasterAxisSynchronized\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "                </config:config-item-map-entry>\n";
        settings_xml << "            </config:config-item-map-indexed>\n";
        settings_xml << "        </config:config-item-set>\n";
        settings_xml << "        <config:config-item-set config:name=\"ooo:configuration-settings\">\n";
        settings_xml << "            <config:config-item config:name=\"ShowZeroValues\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"ShowNotes\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"ShowGrid\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"GridColor\" config:type=\"long\">12632256</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"ShowPageBreaks\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"LinkUpdateMode\" config:type=\"short\">3</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"HasColumnRowHeaders\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"HasSheetTabs\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"IsOutlineSymbolsSet\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"IsSnapToRaster\" config:type=\"boolean\">false</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"RasterIsVisible\" config:type=\"boolean\">false</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"RasterResolutionX\" config:type=\"int\">1000</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"RasterResolutionY\" config:type=\"int\">1000</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"RasterSubdivisionX\" config:type=\"int\">1</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"RasterSubdivisionY\" config:type=\"int\">1</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"IsRasterAxisSynchronized\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"AutoCalculate\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"PrinterName\" config:type=\"string\"/>\n";
        settings_xml << "            <config:config-item config:name=\"PrinterSetup\" config:type=\"base64Binary\"/>\n";
        settings_xml << "            <config:config-item config:name=\"ApplyUserData\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"CharacterCompressionType\" config:type=\"short\">0</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"IsKernAsianPunctuation\" config:type=\"boolean\">false</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"SaveVersionOnClose\" config:type=\"boolean\">false</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"UpdateFromTemplate\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"AllowPrintJobCancel\" config:type=\"boolean\">true</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"LoadReadonly\" config:type=\"boolean\">false</config:config-item>\n";
        settings_xml << "            <config:config-item config:name=\"IsDocumentShared\" config:type=\"boolean\">false</config:config-item>\n";
        settings_xml << "        </config:config-item-set>\n";
        settings_xml << "    </office:settings>\n";
        settings_xml << "</office:document-settings>\n";
        settings_xml.close();
        
        ofstream content_xml((tmp+"/ods_interface/"+FileName+"/content.xml").c_str());
        content_xml << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        content_xml << "<office:document-content\n";
        content_xml << "xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\"\n";
        content_xml << "xmlns:style=\"urn:oasis:names:tc:opendocument:xmlns:style:1.0\"\n";
        content_xml << "xmlns:text=\"urn:oasis:names:tc:opendocument:xmlns:text:1.0\"\n";
        content_xml << "xmlns:table=\"urn:oasis:names:tc:opendocument:xmlns:table:1.0\"\n";
        content_xml << "xmlns:draw=\"urn:oasis:names:tc:opendocument:xmlns:drawing:1.0\"\n";
        content_xml << "xmlns:fo=\"urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0\"\n";
        content_xml << "xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n";
        content_xml << "xmlns:dc=\"http://purl.org/dc/elements/1.1/\"\n";
        content_xml << "xmlns:meta=\"urn:oasis:names:tc:opendocument:xmlns:meta:1.0\"\n";
        content_xml << "xmlns:number=\"urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0\"\n";
        content_xml << "xmlns:presentation=\"urn:oasis:names:tc:opendocument:xmlns:presentation:1.0\"\n";
        content_xml << "xmlns:svg=\"urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0\"\n";
        content_xml << "xmlns:chart=\"urn:oasis:names:tc:opendocument:xmlns:chart:1.0\"\n";
        content_xml << "xmlns:dr3d=\"urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0\"\n";
        content_xml << "xmlns:math=\"http://www.w3.org/1998/Math/MathML\"\n";
        content_xml << "xmlns:form=\"urn:oasis:names:tc:opendocument:xmlns:form:1.0\"\n";
        content_xml << "xmlns:script=\"urn:oasis:names:tc:opendocument:xmlns:script:1.0\"\n";
        content_xml << "xmlns:ooo=\"http://openoffice.org/2004/office\"\n";
        content_xml << "xmlns:ooow=\"http://openoffice.org/2004/writer\"\n";
        content_xml << "xmlns:oooc=\"http://openoffice.org/2004/calc\"\n";
        content_xml << "xmlns:dom=\"http://www.w3.org/2001/xml-events\"\n";
        content_xml << "xmlns:xforms=\"http://www.w3.org/2002/xforms\"\n";
        content_xml << "xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"\n";
        content_xml << "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n";
        content_xml << "xmlns:rpt=\"http://openoffice.org/2005/report\"\n";
        content_xml << "xmlns:of=\"urn:oasis:names:tc:opendocument:xmlns:of:1.2\"\n";
        content_xml << "xmlns:rdfa=\"http://docs.oasis-open.org/opendocument/meta/rdfa#\"\n";
        content_xml << "xmlns:field=\"urn:openoffice:names:experimental:ooo-ms-interop:xmlns:field:1.0\"\n";
        content_xml << "xmlns:formx=\"urn:openoffice:names:experimental:ooxml-odf-interop:xmlns:form:1.0\"\n";
        content_xml << "office:version=\"1.2\">\n";
        content_xml << "    <office:scripts/>\n";
        content_xml << "    <office:font-face-decls>\n";
        content_xml << "        <style:font-face style:name=\"Liberation Sans\"\n";
        content_xml << "            svg:font-family=\"&apos;Liberation Sans&apos;\"\n";
        content_xml << "            style:font-family-generic=\"swiss\"\n";
        content_xml << "            style:font-pitch=\"variable\"/>\n";
        content_xml << "        <style:font-face style:name=\"DejaVu Sans\"\n";
        content_xml << "            svg:font-family=\"&apos;DejaVu Sans&apos;\"\n";
        content_xml << "            style:font-family-generic=\"system\"\n";
        content_xml << "            style:font-pitch=\"variable\"/>\n";
        content_xml << "    </office:font-face-decls>\n";
        content_xml << "    <office:automatic-styles>\n";
        content_xml << "        <style:style style:name=\"co1\" style:family=\"table-column\">\n";
        content_xml << "            <style:table-column-properties fo:break-before=\"auto\" style:column-width=\"2.267cm\"/>\n";
        content_xml << "        </style:style>\n";
        content_xml << "        <style:style style:name=\"ro1\" style:family=\"table-row\">\n";
        content_xml << "            <style:table-row-properties style:row-height=\"0.452cm\"\n";
        content_xml << "                fo:break-before=\"auto\" style:use-optimal-row-height=\"true\"/>\n";
        content_xml << "        </style:style>\n";
        content_xml << "        <style:style style:name=\"ta1\" style:family=\"table\"\n";
        content_xml << "            style:master-page-name=\"Default\">\n";
        content_xml << "            <style:table-properties table:display=\"true\" style:writing-mode=\"lr-tb\"/>\n";
        content_xml << "        </style:style>\n";
        content_xml << "        <style:style style:name=\"ta_extref\" style:family=\"table\">\n";
        content_xml << "            <style:table-properties table:display=\"false\"/>\n";
        content_xml << "        </style:style>\n";
        content_xml << "    </office:automatic-styles>\n";
        content_xml << "    <office:body>\n";
        content_xml << "        <office:spreadsheet>\n";

        for(int Sheet = 0; Sheet < Sheets(); Sheet++)
        {
            content_xml << "            <table:table table:name=\""<<SheetNames[Sheet]<<"\" table:style-name=\"ta"<<Sheet<<"\" table:print=\"false\">\n";
            content_xml << "                <table:table-column table:style-name=\"co1\" table:number-columns-repeated=\""<<Columns(Sheet)<<"\" table:default-cell-style-name=\"Default\"/>\n";
            for(vector<vector<string> >::const_iterator Row = Content[Sheet].begin(); Row != Content[Sheet].end(); Row++)
            {
                content_xml << "                <table:table-row table:style-name=\"ro1\">\n";
                for(vector<string>::const_iterator Cell = Row->begin(); Cell != Row->end(); Cell++)
                {
                    content_xml << "                    <table:table-cell office:value-type=\"string\">\n";
                    content_xml << "                        <text:p>"<< (*Cell) <<"</text:p>\n";
                    content_xml << "                    </table:table-cell>\n";
                }
                content_xml << "                </table:table-row>\n";
            }
            content_xml << "            </table:table>\n";
        }
        
        content_xml << "        </office:spreadsheet>\n";
        content_xml << "    </office:body>\n";
        content_xml << "</office:document-content>\n";
        content_xml.close();
        
        string PWD = getenv("PWD");
        int x = system((string("cd ")+tmp+"/ods_interface/"+FileName+"; zip -q -r "+PWD+"/"+FullFileName+" *").c_str());
        x = system((string("rm -rf ")+tmp+"/ods_interface/"+FileName).c_str());
    }
    
    void Open_Document_Spreadsheet::ExportToHTML(string FileName) const
    {
        ofstream html(FileName.c_str());
        html << "<html>\n\t<body>\n";
        html << "\t\t<h1>" << DocumentTitle << "</h1>\n";
        for(int Sheet = 0; Sheet < Sheets(); Sheet++)
        {
            html << "\t\t<h2>"<<SheetNames[Sheet]<<"</h2><br>\n";
            html << "\t\t<table border=\""<<Border<<"\">\n";
            for(vector<vector<string> >::const_iterator Row = Content[Sheet].begin(); Row != Content[Sheet].end(); Row++)
            {
                html << "\t\t\t<tr>";
                for(vector<string>::const_iterator Cell = Row->begin(); Cell != Row->end(); Cell++)
                {
                    if((*Cell) == "")
                        html << "<td> &nbsp; </td>";
                    else
                        html << "<td> " << (*Cell) << " </td>";
                }
                html << "</tr>\n";
            }
            html << "\t\t</table>\n";
        }
        html << "\t</body>\n<html>\n";
        html.close();
    }

}



